"""Calculate the packet overhead to send the given arguments in SNAP Connect"""
MAX_PKT_SIZE = 123


def get_dmcast_packet_overhead(func, *args):
    """Return the max packet overhead of the given args for a dmcast packet.

    Right now, we just append 3 integers representing  Group, TTL, and Delay
    """
    calc_args = (0, 0, 0) + args  # Account for Group, TTL, and Delay
    return get_packet_overhead(func, *calc_args)


def get_packet_overhead(func, *args):
    """Return the max packet overhead of the given args"""

    # Assume worst case scenario, func isn't a builtin, &c.
    # PID + Dst + Src + Seq + finalDst + origSrc + 3 * # args + 2nd CRC + Packet CRC + func_name P-string
    overhead = 1 + 3 + 3 + 1 + 3 + 3 + 3 * len(args) + 2 + 2 + len(func) + 1
    # p-strings for string/byte args
    strings = sum([1 + len(a) for a in args if isinstance(a, basestring) or isinstance(a, bytearray)])

    return overhead + strings


def get_max_addrs(overhead):
    """Return the maximum number of target addresses that will fit in a packet

    :param int overhead: The overhead of the packet
    """
    return ((MAX_PKT_SIZE - 2) - overhead) / 3
