"""SnapFuture is a special subclass of a Future that is returned from the methods in FSC

It is primarily used to attach additional properties to the future when storing them in a
pending list.
"""
from tornado.concurrent import Future


class SnapFuture(Future):
    """This class inherits from future, but also includes some additional values needed by SNAP Connect"""
    def __init__(self, is_multicast, response_addrs, callback_name):
        """Create a SnapFuture that is resolved when an RPC or Mcast RPC responds

        :param bool is_multicast: True if this is a multicast response
        :param str|list[str] response_addrs: Hexlified addresses we expect to respond to the RPC
        :param str callback_name: Name of the RPC response
        """
        super(SnapFuture, self).__init__()

        self.is_multicast = is_multicast  # True if this is a future for a multicast RPC
        self.responses = {}  # Hang partial responses off of the future
        self.expected_responses = response_addrs
        self.callback_name = callback_name

    def get_key(self):
        """Return a unique key for this future

        This is used as an index into the callback futures dictionary"""
        if self.is_multicast:
            return self.callback_name
        else:
            # RPCs need to be more unique than just the function name
            return "%s.%s" % (self.expected_responses, self.callback_name)
